<?php
/**
 * Core Design Web Gallery plugin for Joomla! 1.5
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @package		Joomla
 * @subpackage	Content
 * @category   	Plugin
 * @version		1.0.0
 * @copyright	Copyright (C)  2007 - 2008 Core Design, http://www.greatjoomla.com
 * @license     http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import library dependencies
jimport('joomla.plugin.plugin');
jimport('joomla.filesystem.file');
// end

class plgContentCdWebGallery extends JPlugin
{

	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @param object $subject The object to observe
	 * @param object $params  The object that holds the plugin parameters
	 * @since 1.5
	 */
	function plgContentCdWebGallery(&$subject)
	{
		parent::__construct($subject);

		// load plugin parameters
		$this->plugin = &JPluginHelper::getPlugin('content', 'cdwebgallery');
		$this->params = new JParameter($this->plugin->params);
	}

	/**
	 * Call the prepare function
	 *
	 * Method is called by the view
	 *
	 * @param 	object		The article object.  Note $article->text is also available
	 * @param 	object		The article params
	 * @param 	int			The 'page' number
	 */
	function onPrepareContent(&$article, &$params, $limitstart=0)
	{

		// define the regular expression for the bot
		$regex = "#{webgallery(?:\s?(.*?)?)?}(.*?){/webgallery}#is";

		if (!preg_match($regex, $article->text)) {
			return;
		}
		
		// define language
		JPlugin::loadLanguage('plg_content_cdwebgallery', JPATH_ADMINISTRATOR);

		// marks replacement
		$article->text = '<div id="webgallery">' . preg_replace_callback($regex, array($this, 'replacer'), $article->text) . '</div>';
	}

	/**
	 * Replacer
	 * @return string
	 */

	function replacer(&$match)
	{
		// $match[0] - whole regular result
		// $match[1] - settings
		// $match[2] - image container - HTML

		// >>> parse $match
		// images
		$image_match = ($match[2] ? trim(strip_tags($match[2], '<img>')) : '');
		// settings
		$settings_match = ($match[1] ? $match[1] : '');
		unset($match);
		// <<<

		// make image array - $image_array
		preg_match_all('(<img.*?>)', $image_match, $image_array);
		$image_array = (isset($image_array[0]) ? $image_array[0] : array());
		unset($image_match);

		// if no images in array, return
		if (!$image_array) {
			return;
		}

		// prevent non-exists images
		$image_array_tmp = array();
		foreach ($image_array as $img) {
			$img_path = JPath::clean($this->absPath() . DS . $this->getAttr($img, 'src'));
			if (JFile::exists($img_path)) {
				$image_array_tmp [] = $img;
			}
		}
		$image_array = $image_array_tmp;

		// >>> define thumbnail variables
		// absolute path to the thumbnail cache foler
		$thumb_cache_dir = $this->absPath() . DS . 'cache';

		// thumb prefix
		$thumb_prefix = 'thumb_webgallery_';
		// <<<

		// load engine class file

		if (preg_match('#engine\s?=\s?"(.*?)"#', $settings_match, $engine)) {
			$engine = trim($engine[1]);
			if (!$this->checkEngine($engine)) {
				$engine = 'popup';
			}
		} else {
			$engine = $this->params->get('engine', 'popup');
			if ($engine == '-1' or !$this->checkEngine($engine))
			{
				$engine = 'popup';
			}
		}

		$class_file = dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'engine' . DS . $engine . DS . 'class.php';

		if (!JFile::exists($class_file)) {
			return;
		}

		// set engine class name
		$classname = $engine . 'WebGalleryEngine';

		// register class
		JLoader::register($classname, $class_file);

		// check the pre-requirements
		$run_engine = '';
		if (is_callable(array($classname, 'preRequirements'))) {
			// call class::requirements() function
			$run_engine = call_user_func(array($classname, 'preRequirements'));
		}
		
		// stop the script if pre-requirements failed
		if ($run_engine) {
			JError::raiseNotice('', JText::_($run_engine));
			return;
		}

		// load theme
		if (preg_match('#theme\s?=\s?"(.*?)"#', $settings_match, $theme)) {
			$theme = trim($theme[1]);
			 
			if (!$this->checkTheme($theme)) {
				$theme = 'default';
			}
			 
		} else {
			$theme = $this->params->get('theme', 'default');
			if ($theme == '-1' or !$this->checkTheme($theme))
			{
				$theme = 'default';
			}
		}
		
		// set title
		$display_gal_title = $this->params->get('display_gal_title', 0);
		
		$title = '';
		
		if ($display_gal_title) {
			if (preg_match('#title\s?=\s?"(.*?)"#', $settings_match)) {
				preg_match('#title\s?=\s?"(.*?)"#', $settings_match, $title);
				$title = (isset($title[1]) ? trim($title[1]) : '');		 
			}
			
			// prevent single gallery titles
			if ($title and $theme != 'single') {
				// sIFR
				$sifr_title = $this->params->get('sifr_title', 0);
				
				// run just once
				static $run_once = 0;						
				if ($sifr_title) {
					
					if (!$run_once) {
						$this->sIFR();
						$run_once = 1;
					}
				}
			}
		}
		
		// display title bellow thumbnail
		$display_alt = $this->params->get('display_alt', 1);

		$i = 0;
		$image_set = array();
		foreach ($image_array as $image)
		{
			$image_set[$i]->img_tag = $image;
			$image_set[$i]->img_src = $this->getAttr($image, 'src');
			$image_set[$i]->img_src_path = JPath::clean($this->absPath() . DS . $image_set[$i]->img_src);
			if ($display_alt) {
				$image_set[$i]->img_alt = $this->getAttr($image, 'alt');
			} else {
				$image_set[$i]->img_alt = '';
			}
			$image_set[$i]->img_title = $this->getAttr($image, 'title');

			$i++;
		}

		// set watermark
		if (preg_match('#watermark\s?=\s?"(yes|no)"#', $settings_match, $watermark)) {
			$watermark = trim($watermark[1]);
			 
		} else {
			$watermark = $this->params->get('watermark', 'yes');
		}

		$watermark = ($watermark == 'yes' ? 1 : 0);

		// create a thumbnail cache
		$thumbnail_array = $this->createThumbnails($image_set, $thumb_cache_dir, $thumb_prefix, $watermark);

		$i = 0;
		foreach ($thumbnail_array as $thumb)
		{
			$image_set[$i]->thumb_name = $thumb;
			$image_set[$i]->thumb_path = $thumb_cache_dir . DS . $image_set[$i]->thumb_name;
			$image_set[$i]->thumb_src = JURI::root(true) . '/cache/' . $image_set[$i]->thumb_name;

			$thumb_size = getimagesize($image_set[$i]->thumb_path);
			$image_set[$i]->thumb_width = (isset($thumb_size[0]) ? $thumb_size[0] : '');
			$image_set[$i]->thumb_height = (isset($thumb_size[1]) ? $thumb_size[1] : '');

			$i++;
		}

		// save random id to session
		$this->saveOptions(count($image_set), $title, $theme);
		
		// call ENGINE::header() function if is possible
		if (is_callable(array($classname, 'header'))) {
			call_user_func(array($classname, 'header'));
		}

		$html_tag = ''; // initiate variable
		
		// call class::tag() function - the calling is  necessary
		$tag = call_user_func(array($classname, 'tag'), $image_set);
		
		// add theme to A and IMG tags
		$html_tag .= $this->loadTheme($theme, $tag);

		return $html_tag;

	}
	
	/*
	 * Load sIFR
	 */
	function sIFR() {
		$document = &JFactory::getDocument(); // set document for next usage
		$live_path_abs = JURI::root(true) . '/';
		// add script
		$document->addScript($live_path_abs .
            'plugins/content/cdwebgallery/utils/js/sifr/jquery.flash.min.js');
		$document->addScript($live_path_abs .
            'plugins/content/cdwebgallery/utils/js/sifr/jquery.sifr.min.js');
		
		$params = call_user_func(array('plgContentCdWebGallery', 'parseIni'), dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'utils' . DS . 'js' . DS . 'sifr' . DS . 'params.ini');
		$font = $params->get('font', 'Garamond');
		$width = $params->get('width', 500);
		$height = $params->get('height', 30);
		$color = $params->get('color', '#888888');
		
		$script = "
		<script type=\"text/javascript\">
			<!--
			jQuery(function($) {
				$('#webgallery h3.title').sifr({
					path: 'plugins/content/cdwebgallery/utils/js/sifr/fonts/',
					save: true,
					font: '$font',
					width: $width,
					height: $height,
					color: '$color'
				});
			});
			-->
		</script>\n";
		$document->addCustomTag($script);
	}

	/*
	 * Check it user inserted $theme (via preg_match function) exists
	 * @return boolean
	 */
	function checkTheme($theme = '') {
		if (!$theme) {
			return false;
		}
		jimport('joomla.filesystem.folder');
		$list = JFolder::folders(dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'theme');

		if (in_array($theme, $list)) {
			return true;
		} else {
			return false;
		}
	}

	/*
	 * Check it user inserted $engine (via preg_match function) exists
	 * @return boolean
	 */
	function checkEngine($engine) {
		if (!$engine) {
			return false;
		}
		jimport('joomla.filesystem.folder');
		$list = JFolder::folders(dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'engine');

		if (in_array($engine, $list)) {
			return true;
		} else {
			return false;
		}
	}

	/*
	 * Create thumbnails
	 * @return array
	 */
	function createThumbnails($image_set, $thumb_abs_path, $thumb_prefix = 'thumb_webgallery_', $watermark = 1) {

		// load thumbnail.class.php file
		require_once(dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'utils' . DS . 'php' . DS . 'webgallery.class.php');

		$outputFormat = (string)$this->params->get('thumb_output', 'JPG');
		$jpg_quality = $this->params->get('jpg_quality', 70);

		$thumb_width = $this->params->get('thumb_width', 100);
		$thumb_height = $this->params->get('thumb_height', 100);

		$watermark_image = $this->params->get('watermark_name', 'magnify.png');
		$watermark_image_path = dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'images' . DS . $watermark_image;
		$watermark_valign = $this->params->get('watermark_valign', 'BOTTOM');
		$watermark_halign = $this->params->get('watermark_halign', 'RIGHT');

		$thumbnail_name_array = array();

		foreach ($image_set as $image) {
			$image_src = $image->img_src;
				
			$image_filename = JFile::stripExt(basename($image_src));
				
			$image_abs_path = $image->img_src_path;
				
				
			$thumbnail_name = $thumb_prefix . $thumb_width . 'x' . $thumb_height . '_watermark-' . $watermark . '_' . $image_filename . '.' . strtolower($outputFormat);

			$thumbnail_path = $thumb_abs_path . DS . $thumbnail_name;
				
			// if thumbnail already exists, return
			if (!JFile::exists($thumbnail_path)) {
				// create a thumbnail
				$thumb_class = new webGalleryThumbnail($image_abs_path);
				$thumb_class->output_format = $outputFormat;

				if (strtolower($outputFormat) == 'jpg') {
					$thumb_class->quality = $jpg_quality;
				}

				if ($watermark) {
					$thumb_class->img_watermark = $watermark_image_path;
					$thumb_class->img_watermark_Valing = $watermark_valign;
					$thumb_class->img_watermark_Haling = $watermark_halign;
				}

				$thumb_class->size_width($thumb_width); // thumbnail width
				$thumb_class->size_height($thumb_height); // thumbnail height
				$thumb_class->process(); // create thumbnail

				$status = $thumb_class->save($thumbnail_path); // define status
			}
				
			$thumbnail_name_array []= $thumbnail_name;
		}
		return $thumbnail_name_array;
	}

	/*
	 * Get attribute from HTML tag (based on regular expression)
	 */
	function getAttr($html_tag, $attr) {
		preg_match('#' . $attr . '\s?=\s?"(.*?)"#', $html_tag, $attr_value);
		if (isset($attr_value[1])) {
			return $attr_value[1];
		} else {
			return;
		}
	}

	/*
	 * Return absolute path to Joomla! root
	 */
	function absPath() {
		return dirname(dirname(dirname(__FILE__)));
	}

	/*
	 * Retrieve plugin parameters
	 * @return string
	 */
	function pluginParams($param = '', $def_val = '') {

		if (!$param or !$def_val) {
			return;
		}

		$folder = 'content';
		$plg_name = 'cdwebgallery';

		// load plugin parameters
		$plugin = &JPluginHelper::getPlugin($folder, $plg_name);
		$pluginParams = new JParameter($plugin->params);

		return $pluginParams->get($param, $def_val);
	}

	/*
	 * Load theme and return IMG and A tags
	 * @return string
	 */
	function loadTheme($theme = 'default', $tag = array()) {

		$document = &JFactory::getDocument(); // set document for next usage
		$compress = $this->params->get('compress', 1);

		$live_path_abs = JURI::root(true) . '/';

		// add CSS
		$document->addStyleSheet($live_path_abs .
			            'plugins/content/cdwebgallery/theme/' . $theme . '/css/cssloader.php?compress=' . $compress . '', 'text/css');

		// set helper.php file
		$theme_class_file = dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'theme' . DS . $theme . DS . 'class.php';

		if (!JFile::exists($theme_class_file)) {
			$theme = 'default';
			// set helper.php file
			$theme_class_file = dirname(__FILE__) . DS . 'cdwebgallery' . DS . 'theme' . DS . $theme . DS . 'class.php';
		}

		$classname = $theme . 'WebGalleryTheme';
		JLoader::register($classname, $theme_class_file);
		
		// check the pre-requirements
		$run_theme = '';
		if (is_callable(array($classname, 'preRequirements'))) {
			// call class::requirements() function
			$run_theme = call_user_func(array($classname, 'preRequirements'));
		}

		// stop the script if pre-requirements failed
		if ($run_theme) {
			JError::raiseNotice('', JText::_($run_theme));
			return;
		}
		
		// call THEME::header() function
		if (is_callable(array($classname, 'header'))) {
			call_user_func(array($classname, 'header'), $theme);
		}
		// call THEME::template() function
		$themed_tag = call_user_func(array($classname, 'template'), $tag);
		return $themed_tag;
	}

	/*
	 * Save random id session
	 * @return void
	 */
	function saveOptions($count = 0, $title = '', $theme = 'default') {

		$session = &JFactory::getSession();
		$session_name = 'cdWebGalleryOptions';
		
		if ($count > 1) {
			$mode = 'gallery';
		} else {
			$mode = 'single';
		}
		
		$random = mt_rand(1, 1000);
		
		jimport('joomla.helper.arrayhelper');
		
		$gallery_settings = array();
		$gallery_settings['title'] = $title;
		$gallery_settings['theme'] = $theme;
		$gallery_settings['mode'] = $mode;
		$gallery_settings['id'] = $random;
		$obj = JArrayHelper::toObject($gallery_settings, 'stdClass');
		
		$session->set($session_name, $obj);
	}

	/*
	 * Retrieve random options from session
	 * @return integer, string or object
	 */
	function options($mode = 'id') {
		$session = &JFactory::getSession();
		$session_name = 'cdWebGalleryOptions';
		$session_obj = $session->get($session_name);
		
		$random = null;
		
		if (!is_null($session_obj)) {
			switch ($mode) {
				default:
					if (isset($session_obj->$mode)) {
						$random = $session_obj->$mode;
					}
					break;
				case '':
				case 'all':
					$random = (object) $session_obj;
					break;
			}
		}
		return $random;
	}

	/*
	 * Routine to check Scriptegrator plugin
	 * @return string
	 */
	function checkScriptegrator($version_number = '1.3.3', $library = 'jquery', $place = 'site') {
		$message = '';
		// check if Scriptegrator is enabled
		if (!defined('_JSCRIPTEGRATOR'))
		{
			$message = JText::_('CDWEBGALLERY_ENABLE_SCRIPTEGRATOR');
		} else {
			// check version
			$version = $version_number;
			if (!JScriptegrator::versionRequire($version))
			{
				$message = JText::sprintf('CDWEBGALLERY_SCRIPTEGRATOR_REQUIREVERSION', $version);
			} else {
				// check place of library
				if (!JScriptegrator::checkLibrary($library, $place))
				{
					$message = JText::_('CDWEBGALLERY_MISSING_' . strtoupper($library));
				}
			}
		}
		return $message;
	}
	
	/*
	 * Parse INI file and load the settings from file
	 * @return object
	 */
	function parseIni($path_to_ini) {
		if (JFile::exists($path_to_ini)) {
			$data = JFile::read($path_to_ini);
		} else  {
			$data = null;
		}
		
		return new JParameter($data);
	}
}

?>
